"""Backward compatibility shim — redirects to aauth_header.py."""

from .aauth_header import (
    parse_agent_auth_header,
    build_agent_auth_challenge,
    parse_aauth_header,
    parse_aauth_requirement,
    parse_signature_requirement,
    build_pseudonym_challenge,
    build_pseudonym_requirement,
    build_identity_challenge,
    build_identity_requirement,
    build_auth_token_challenge,
    build_auth_token_requirement,
    build_interaction_challenge,
    build_interaction_requirement,
    build_approval_challenge,
    build_approval_requirement,
    build_aauth_error,
    build_signature_error,
    parse_aauth_error,
    parse_signature_error,
    REQUIRE_PSEUDONYM,
    REQUIRE_IDENTITY,
    REQUIRE_AUTH_TOKEN,
    REQUIRE_INTERACTION,
    REQUIRE_APPROVAL,
)

__all__ = [
    "parse_agent_auth_header",
    "build_agent_auth_challenge",
    "parse_aauth_header",
    "parse_aauth_requirement",
    "parse_signature_requirement",
    "build_pseudonym_challenge",
    "build_pseudonym_requirement",
    "build_identity_challenge",
    "build_identity_requirement",
    "build_auth_token_challenge",
    "build_auth_token_requirement",
    "build_interaction_challenge",
    "build_interaction_requirement",
    "build_approval_challenge",
    "build_approval_requirement",
    "build_aauth_error",
    "build_signature_error",
    "parse_aauth_error",
    "parse_signature_error",
    "REQUIRE_PSEUDONYM",
    "REQUIRE_IDENTITY",
    "REQUIRE_AUTH_TOKEN",
    "REQUIRE_INTERACTION",
    "REQUIRE_APPROVAL",
]
